@@ -18,6 +18,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h" //For VERSION, GLOM_ENABLE_CLIENT_ONLY, GLOM_ENABLE_SQLITE
+
 #include "application.h"
 #include "dialog_existing_or_new.h"
 
@@ -35,8 +37,6 @@
 #include <glom/libglom/connectionpool_backends/postgres_central.h>
 #include <glom/libglom/connectionpool_backends/postgres_self.h>
 
-#include "config.h" //For VERSION.
-
 #include <cstdio>
 #include <memory> //For std::auto_ptr<>
 #include <giomm.h>
@@ -1077,6 +1077,8 @@ bool App_Glom::on_document_load()
             {
               //Let the user try again.
               //A warning has already been shown.
+              //TODO: No, I don't think there is a warning.
+              std::cerr << "App_Glom::on_document_load(): recreate_database() failed." << std::endl;
               return offer_new_or_existing();
             }
             else
@@ -1222,10 +1224,8 @@ bool App_Glom::offer_new_or_existing()
         open_browsed_document(dialog->get_service_info(), dialog->get_service_name());
         break;
 #endif
-#ifndef GLOM_ENABLE_CLIENT_ONLY
       default:
         break;
-#endif // !GLOM_ENABLE_CLIENT_ONLY
       }
 
       //Check that a document was opened:
@@ -1938,13 +1938,13 @@ void App_Glom::on_menu_file_save_as_example()
 
         //std::cout << "debug: table_name=" << table_name << std::endl;
 
-        Glib::ustring row_text;
+        Document_Glom::type_example_rows example_rows;
         FoundSet found_set;
         found_set.m_table_name = table_name;
-        m_pFrame->export_data_to_string(row_text, found_set, sequence);
+        m_pFrame->export_data_to_vector(example_rows, found_set, sequence);
         //std::cout << "  debug after row_text=" << row_text << std::endl;
 
-        document->set_table_example_data(table_name, row_text);
+        document->set_table_example_data(table_name, example_rows);
       }
 
       document->set_allow_autosave(true);
@@ -2219,12 +2219,18 @@ Glib::ustring App_Glom::ui_file_select_save(const Glib::ustring& old_file_uri) /
           continue;
         }
       }
- 
+
+      bool is_self_hosted = false;
+      if(m_ui_save_extra_newdb_hosting_mode == Document_Glom::POSTGRES_SELF_HOSTED)
+        is_self_hosted = true;
+#ifdef GLOM_ENABLE_SQLITE
+      if(m_ui_save_extra_newdb_hosting_mode == Document_Glom::SQLITE_HOSTED)
+        is_self_hosted = true;
+#endif // GLOM_ENABLE_SQLITE
+
       // Create a directory for self-hosted databases (sqlite or self-hosted
       // postgresql).
-      if(!try_again && fileChooser_SaveExtras &&
-         (m_ui_save_extra_newdb_hosting_mode == Document_Glom::POSTGRES_SELF_HOSTED ||
-          m_ui_save_extra_newdb_hosting_mode == Document_Glom::SQLITE_HOSTED))
+      if(!try_again && fileChooser_SaveExtras && is_self_hosted)
       {
         //Check that the directory does not exist already.
         //The GtkFileChooser could not check for that because it could not know that we would create a directory based on the filename:
