@@ -157,7 +157,10 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   m_iter_new_template = m_new_model->append();
   (*m_iter_new_template)[m_new_columns.m_col_title] = _("New From Template");
 
-  // Load example files
+
+ // Load example files:
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+
 #ifdef G_OS_WIN32
   gchar* dir = g_win32_get_package_installation_subdirectory(NULL, NULL, "share/glom/doc/examples");
   std::string path(dir);
@@ -166,25 +169,21 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   if(!Glib::file_test(path, Glib::FILE_TEST_EXISTS))
     path = GLOM_EXAMPLES_DIR;
 #else
-#ifndef GLOM_ENABLE_CLIENT_ONLY
   const char* path = GLOM_EXAMPLES_DIR;
-#endif
-#endif
+#endif //G_OS_WIN32
 
-#ifndef GLOM_ENABLE_CLIENT_ONLY
-  m_examples_dir = Gio::File::create_for_path(path);
+  if(!list_examples_at_path(path))
+  {
+    // If that directory did not exist, then try this one instead:
+    // (An Ubuntu package puts the example files here for some reason.
+    // TODO: Find out what Makefile.am variable to use to just use this automatically instead.
+    #ifndef G_OS_WIN32
+    list_examples_at_path(GLOM_EXAMPLES_DIR_ALTERNATIVE);
+    #endif //G_OS_WIN32
+  }
+
+#endif //!GLOM_ENABLE_CLIENT_ONLY
 
-  try
-  {
-    m_examples_dir->enumerate_children_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_enumerate_children),
-                                             G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE","G_FILE_ATTRIBUTE_STANDARD_NAME);
-    // TODO: Monitor example directory for new/removed files?
-  }
-  catch(const Glib::Exception& ex)
-  {
-    std::cerr << "Could not enumerate examples: " << ex.what() << std::endl;
-  }
-#endif /* !GLOM_ENABLE_CLIENT_ONLY */
 
   // Browse local network
 #ifndef G_OS_WIN32
@@ -240,12 +239,34 @@ Dialog_ExistingOrNew::Dialog_ExistingOrNew(BaseObjectType* cobject, const Glib::
   new_view_selection->set_select_function( sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_new_select_func) );
 #else /* GLOM_ENABLE_CLIENT_ONLY */
   m_notebook->remove_page(NEW_PAGE);
-  m_notebook->set_show_tabs (false);
+  m_notebook->set_show_tabs(false);
 #endif /* !GLOM_ENABLE_CLIENT_ONLY */
     
   update_ui_sensitivity();
 }
 
+#ifndef GLOM_ENABLE_CLIENT_ONLY
+bool Dialog_ExistingOrNew::list_examples_at_path(const std::string& path)
+{
+  std::cout << "Dialog_ExistingOrNew::list_examples_at_path(): path=" << path << std::endl;
+
+  m_examples_dir = Gio::File::create_for_path(path);
+
+  try
+  {
+    m_examples_dir->enumerate_children_async(sigc::mem_fun(*this, &Dialog_ExistingOrNew::on_enumerate_children),
+                                             G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE","G_FILE_ATTRIBUTE_STANDARD_NAME);
+    // TODO: Monitor example directory for new/removed files?
+    return true;
+  }
+  catch(const Glib::Exception& ex)
+  {
+    std::cerr << "Could not enumerate examples at path=:" << path << ex.what() << std::endl;
+    return false;
+  }
+}
+#endif // !GLOM_ENABLE_CLIENT_ONLY
+
 Dialog_ExistingOrNew::~Dialog_ExistingOrNew()
 {
 #ifndef G_OS_WIN32
@@ -277,7 +298,7 @@ Dialog_ExistingOrNew::~Dialog_ExistingOrNew()
   }
 }
 
-bool Dialog_ExistingOrNew::on_existing_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool path_currently_selected)
+bool Dialog_ExistingOrNew::on_existing_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool /* path_currently_selected */)
 {
   Gtk::TreeModel::iterator iter = model->get_iter(path);
 #ifndef G_OS_WIN32
@@ -290,7 +311,7 @@ bool Dialog_ExistingOrNew::on_existing_select_func(const Glib::RefPtr<Gtk::TreeM
   return true;
 }
 
-bool Dialog_ExistingOrNew::on_new_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool path_currently_selected)
+bool Dialog_ExistingOrNew::on_new_select_func(const Glib::RefPtr<Gtk::TreeModel>& model, const Gtk::TreeModel::Path& path, bool /* path_currently_selected */)
 {
   Gtk::TreeModel::iterator iter = model->get_iter(path);
   if(iter == m_iter_new_template)
@@ -515,7 +536,7 @@ void Dialog_ExistingOrNew::new_title_data_func(Gtk::CellRenderer* renderer, cons
   }
 }
 
-void Dialog_ExistingOrNew::on_switch_page(GtkNotebookPage* page, guint page_num)
+void Dialog_ExistingOrNew::on_switch_page(GtkNotebookPage* /* page */, guint /* page_num */)
 {
   update_ui_sensitivity();
 }
@@ -578,6 +599,9 @@ void Dialog_ExistingOrNew::update_ui_sensitivity()
 #ifndef GLOM_ENABLE_CLIENT_ONLY
 void Dialog_ExistingOrNew::on_enumerate_children(const Glib::RefPtr<Gio::AsyncResult>& res)
 {
+  if(!m_examples_dir)
+    return;
+
   try
   {
     m_examples_enumerator = m_examples_dir->enumerate_children_finish(res);
@@ -585,7 +609,7 @@ void Dialog_ExistingOrNew::on_enumerate_children(const Glib::RefPtr<Gio::AsyncRe
   }
   catch(const Glib::Exception& ex)
   {
-    std::cerr << "Could not enumerate examples: " << ex.what() << std::endl;
+    std::cerr << "Could not enumerate files in examples directory: " << ex.what() << std::endl;
     
     m_examples_enumerator.reset();
     m_examples_dir.reset();
@@ -597,7 +621,7 @@ void Dialog_ExistingOrNew::on_next_files(const Glib::RefPtr<Gio::AsyncResult>& r
   try
   {
     const Glib::ListHandle<Glib::RefPtr<Gio::FileInfo> >& list = m_examples_enumerator->next_files_finish(res);
-    if (list.empty())
+    if(list.empty())
     {
       // Done
       m_examples_dir.reset();
@@ -699,6 +723,16 @@ void Dialog_ExistingOrNew::on_stream_read(const Glib::RefPtr<Gio::AsyncResult>&
 #endif /* !GLOM_ENABLE_CLIENT_ONLY */
 
 #ifndef G_OS_WIN32
+void Dialog_ExistingOrNew::on_service_found_static(EpcServiceMonitor* /* monitor */, gchar* name, EpcServiceInfo* info, gpointer user_data)
+{
+  static_cast<Dialog_ExistingOrNew*>(user_data)->on_service_found(name, info);
+}
+
+void Dialog_ExistingOrNew::on_service_removed_static(EpcServiceMonitor* /* monitor */, gchar* name, gchar* type, gpointer user_data)
+{
+  static_cast<Dialog_ExistingOrNew*>(user_data)->on_service_removed(name, type);
+}
+
 void Dialog_ExistingOrNew::on_service_found(const Glib::ustring& name, EpcServiceInfo* info)
 {
   //Translator hint: This is <Service Name> on <Host> (via Network Interface such as eth0).
@@ -720,7 +754,7 @@ void Dialog_ExistingOrNew::on_service_found(const Glib::ustring& name, EpcServic
   }
 }
 
-void Dialog_ExistingOrNew::on_service_removed(const Glib::ustring& name, const Glib::ustring& type)
+void Dialog_ExistingOrNew::on_service_removed(const Glib::ustring& name, const Glib::ustring& /* type */)
 {
   // Find the entry with the given name
   const Gtk::TreeNodeChildren& children = m_iter_existing_network->children();
@@ -742,7 +776,7 @@ void Dialog_ExistingOrNew::on_service_removed(const Glib::ustring& name, const G
 }
 #endif // !G_OS_WIN32
 
-void Dialog_ExistingOrNew::on_existing_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column)
+void Dialog_ExistingOrNew::on_existing_row_activated(const Gtk::TreeModel::Path& /* path */, Gtk::TreeViewColumn* /* column */)
 {
   if(m_select_button->is_sensitive())
     on_select_clicked();
@@ -756,7 +790,7 @@ void Dialog_ExistingOrNew::on_existing_button_clicked(const Gtk::TreeModel::Path
     on_select_clicked();
 }
 
-void Dialog_ExistingOrNew::on_new_row_activated(const Gtk::TreeModel::Path& path, Gtk::TreeViewColumn* column)
+void Dialog_ExistingOrNew::on_new_row_activated(const Gtk::TreeModel::Path& /* path */, Gtk::TreeViewColumn* /* column */)
 {
   if(m_select_button->is_sensitive())
     on_select_clicked();
@@ -799,4 +833,5 @@ void Dialog_ExistingOrNew::on_select_clicked()
   }
 }
 
+
 } //namespace Glom
