@@ -24,6 +24,9 @@
 #include <glom/libglom/utils.h>
 #include <glibmm/i18n.h>
 
+namespace Glom
+{
+
 //Initialize static data:
 Field::type_map_gda_type_to_glom_type Field::m_map_gda_type_to_glom_type;
 Field::type_map_glom_type_to_gda_type Field::m_map_glom_type_to_gda_type;
@@ -35,7 +38,10 @@ bool Field::m_maps_inited = false;
 
 Field::Field()
 : m_glom_type(TYPE_INVALID),
-  m_visible(true)
+  m_field_info(Gnome::Gda::Column::create()),
+  m_visible(true),
+  m_primary_key(false),
+  m_unique_key(false)
 {
   m_translatable_item_type = TRANSLATABLE_TYPE_FIELD;
 }
@@ -56,8 +62,7 @@ Field& Field::operator=(const Field& src)
   TranslatableItem::operator=(src);
 
   m_glom_type = src.m_glom_type;
-  m_field_info = src.m_field_info;
-  m_data = src.m_data;
+  m_field_info = src.m_field_info->copy();
 
   m_lookup_relationship = src.m_lookup_relationship;
   m_strLookupField = src.m_strLookupField;
@@ -66,6 +71,9 @@ Field& Field::operator=(const Field& src)
 
   m_visible = src.m_visible;
 
+  m_primary_key = src.m_primary_key;
+  m_unique_key = src.m_unique_key;
+
   m_default_formatting = src.m_default_formatting;
 
   return *this;
@@ -74,13 +82,14 @@ Field& Field::operator=(const Field& src)
 bool Field::operator==(const Field& src) const
 {
   return TranslatableItem::operator==(src)
-         && (m_field_info == src.m_field_info)
+         && (m_field_info->equal(src.m_field_info))
          && (m_glom_type == src.m_glom_type)
-         && (m_data == src.m_data)
          && (m_lookup_relationship == src.m_lookup_relationship)
          && (m_strLookupField == src.m_strLookupField)
          && (m_calculation == src.m_calculation)
          && (m_visible == src.m_visible)
+         && (m_primary_key == src.m_primary_key)
+         && (m_unique_key == src.m_unique_key)
          && (m_default_formatting == src.m_default_formatting);
 }
 
@@ -101,30 +110,83 @@ Field::glom_field_type Field::get_glom_type() const
 
 void Field::set_glom_type(glom_field_type fieldtype)
 {
+  glom_field_type old_type = m_glom_type;
+
+  // Set glom type from fieldinfo if it represents a different type.
   m_glom_type = fieldtype;
+
+  // Reset default value if the default value type does not match the new
+  // glom type.
+  // TODO: Should we try to convert the default value to the new type here?
+  if(fieldtype != old_type)
+    set_default_value(Gnome::Gda::Value()); //TODO: Try to convert it, maybe by rendering and parsing it.
 }
 
-Gnome::Gda::FieldAttributes Field::get_field_info() const
+Glib::RefPtr<Gnome::Gda::Column> Field::get_field_info()
 {
   return m_field_info;
 }
 
-void Field::set_field_info(const Gnome::Gda::FieldAttributes& fieldInfo)
+Glib::RefPtr<const Gnome::Gda::Column> Field::get_field_info() const
 {
-  m_field_info = fieldInfo;
-
-  //TODO: Maybe just do this in get_glom_type()?
-  set_glom_type( get_glom_type_for_gda_type(fieldInfo.get_gdatype()) );
+  return m_field_info;
 }
 
-Gnome::Gda::Value Field::get_data() const
+void Field::set_field_info(const Glib::RefPtr<Gnome::Gda::Column>& fieldinfo)
 {
-  return m_data;
-}
+  m_field_info = fieldinfo;
 
-void Field::set_data(const Gnome::Gda::Value& data)
-{
-  m_data = data;
+  // Also take fallback types into account as fieldinfo might originate from
+  // the database system directly.
+  GType new_type = fieldinfo->get_g_type();
+
+  GType cur_type = G_TYPE_NONE;
+  if(get_glom_type() != TYPE_INVALID)
+  {
+    cur_type = get_gda_type_for_glom_type(get_glom_type());
+
+    const FieldTypes* pFieldTypes = NULL;
+
+    ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
+    if(pConnectionPool)
+      pFieldTypes = pConnectionPool->get_field_types();
+
+    if(pFieldTypes)
+    {
+      while(cur_type != new_type && cur_type != G_TYPE_NONE)
+        cur_type = pFieldTypes->get_fallback_type_for_gdavaluetype(cur_type);
+    }
+  }
+
+  if(cur_type == G_TYPE_NONE)
+    set_glom_type( get_glom_type_for_gda_type(fieldinfo->get_g_type()) );
+
+  Gnome::Gda::Value value = get_default_value();
+  if(!value.is_null())
+  {
+    // Check that the default value is consistent with the field's type
+    // TODO: Basically copied from set_default_value(). Maybe this check should
+    // be moved into an extra function.
+    GType cur_type = get_gda_type_for_glom_type(get_glom_type());
+    const FieldTypes* pFieldTypes = NULL;
+
+    ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
+    if(pConnectionPool)
+      pFieldTypes = pConnectionPool->get_field_types();
+
+    // Take into account that value might be one of the fallback types
+    if(pFieldTypes)
+    {
+      while(cur_type != value.get_value_type() && cur_type != G_TYPE_NONE)
+        cur_type = pFieldTypes->get_fallback_type_for_gdavaluetype(cur_type);
+    }
+
+    if(!value.is_null() && value.get_value_type() != cur_type)
+    {
+      g_warning("Field::set_field_info: New field's default value type (%s) does not match field type (%s). Resetting default value.", g_type_name(value.get_value_type()), g_type_name(get_gda_type_for_glom_type(get_glom_type())));
+      m_field_info->set_default_value(Gnome::Gda::Value());
+    }
+  }
 }
 
 sharedptr<Relationship> Field::get_lookup_relationship() const
@@ -153,190 +215,123 @@ bool Field::get_is_lookup() const
   return ( m_lookup_relationship && (!m_strLookupField.empty()) );
 }
 
-namespace { //anonymous
-
-//A copy of PQescapeString from the Postgres source, to avoid linking with libpql directly, 
-//and to use until we can use the latest libgda, which has an equivalent.
-
-#define SQL_STR_DOUBLE(ch)	((ch) == '\'' || (ch) == '\\')
-
-/*
- * Escaping arbitrary strings to get valid SQL literal strings.
- *
- * Replaces "\\" with "\\\\" and "'" with "''".
- *
- * length is the length of the source string.  (Note: if a terminating NUL
- * is encountered sooner, PQescapeString stops short of "length"; the behavior
- * is thus rather like strncpy.)
- *
- * For safety the buffer at "to" must be at least 2*length + 1 bytes long.
- * A terminating NUL character is added to the output string, whether the
- * input is NUL-terminated or not.
- *
- * Returns the actual length of the output (not counting the terminating NUL).
- */
-size_t
-Glom_PQescapeString(char *to, const char *from, size_t length)
+Glib::ustring Field::sql(const Gnome::Gda::Value& value, const Glib::RefPtr<Gnome::Gda::Connection>& connection) const
 {
-	const char *source = from;
-	char	   *target = to;
-	size_t		remaining = length;
+  //g_warning("Field::sql: glom_type=%d", get_glom_type());
 
-	while (remaining > 0 && *source != '\0')
-	{
-		if (SQL_STR_DOUBLE(*source))
-			*target++ = *source;
-		*target++ = *source++;
-		remaining--;
-	}
-
-	/* Write the terminating NUL character. */
-	*target = '\0';
-
-	return target - to;
-}
-
-} //anonymous
-
-static std::string glom_escape_text(const std::string src)
-{
-  if(src.empty())
+  if(value.is_null() && (get_glom_type() == TYPE_TEXT))
+  {
     return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
+  }
+
+
+  //Use libgda's DataHandler to get the string for SQL:
+  Glib::RefPtr<const Gnome::Gda::ServerProvider> provider = connection->get_provider();
+  if(!provider)
+  {
+    std::cerr << "Field::sql(): The ServerProvider was null." << std::endl;
+    return Glib::ustring();
+  } 
+
+  const GType gda_type = get_gda_type_for_glom_type(m_glom_type);
+  Glib::RefPtr<const Gnome::Gda::DataHandler> datahandler = 
+    provider->get_data_handler_g_type(connection, gda_type);
+  if(datahandler)
+  {
+    //Note that this does seems to add the needed quotes too, for instance around strings,
+    //though that is not clearly documented. See bug http://bugzilla.gnome.org/show_bug.cgi?id=572220
+    return datahandler->get_sql_from_value(value);
+  }
   else
   {
-    const size_t len = src.size();
-    char* to = (char*)malloc(sizeof(char) * 2 * (len + 1)); //recommended size for to.
-    const size_t len_escaped = Glom_PQescapeString(to, src.c_str(), len);
-    if(!len_escaped)
-    {
-      std::cerr << "glom_escape_text(): Glom_PQescapeString() failed with text: " << src << std::endl;
-
-      if(to)
-        free(to);
-
-      return "''";
-    }
-    else
-    {
-      std::string escaped(to, len_escaped);
-      free(to);
-
-      //Also escape any ";" characters, because these also cause problems, at least with libgda:
-      //See bug #326325.
-      escaped = GlomUtils::string_replace(escaped, ";", "\\073");
-
-      return ("'" + escaped + "'"); //Add single-quotes. Actually escape it 
-      //std::cout << "glom_escape_text: escaped and quoted: " << str << std::endl;
-    }
+    std::cerr << "Field::sql(): The DataHandler was null." << std::endl;
+    return Glib::ustring();
   }
+
+  return Glib::ustring();
 }
 
 Glib::ustring Field::sql(const Gnome::Gda::Value& value) const
 {
-  //g_warning("Field::sql: glom_type=%d", get_glom_type());
-
-  if(value.is_null())
+  //TODO: Handle exceptions as in BaseDB::connect().
+  sharedptr<SharedConnection> connection = ConnectionPool::get_instance()->connect();
+  if(connection)
   {
-    switch(get_glom_type())
-    {
-      case(TYPE_TEXT):
-      {
-        return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
-        break;
-      }
-      case(TYPE_DATE):
-      case(TYPE_TIME):
-      case(TYPE_NUMERIC):
-      case(TYPE_IMAGE):
-      {
-        return "NULL";
-        break;
-      }
-      case(TYPE_INVALID):
-      {
-        g_warning("Field::sql(): The field type is INVALID.");
-        return "NULL";
-        break;
-      }
-      default:
-      {
-        //Don't deal with these here.
-        break;
-      }
-    }
+    Glib::RefPtr<Gnome::Gda::Connection> gda_connection = connection->get_gda_connection();
+    if(gda_connection)
+      return sql(value, gda_connection);
   }
 
-  Glib::ustring str;
+  return Glib::ustring();
+}
 
-  switch(get_glom_type())
+Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value) const
+{
+  return to_file_format(value, m_glom_type);
+}
+
+Glib::ustring Field::to_file_format(const Gnome::Gda::Value& value, glom_field_type glom_type)
+{
+  if(glom_type == TYPE_IMAGE)
   {
-    case(TYPE_TEXT):
+    if(!value.gobj() || (value.get_value_type() != GDA_TYPE_BINARY))
+      return Glib::ustring();
+ 
+    const GdaBinary* gdabinary = gda_value_get_binary(value.gobj());
+    if(!gdabinary)
+      return Glib::ustring();
+    else
     {
-      if(value.is_null())
-        return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
-      else
-      {
-        str = glom_escape_text(value.get_string());
-      }
-
-      break;
-    }
-    case(TYPE_DATE):
-    case(TYPE_TIME):
-    {
-      NumericFormat format_ignored; //Because we use ISO format.
-      str = GlomConversions::get_text_for_gda_value(m_glom_type, value, std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
-      if(str != "NULL")
-         str = "'" + str + "'"; //Add single-quotes.
-
-      break;
-    }
-    case(TYPE_NUMERIC):
-    {
-      str =  GlomConversions::get_text_for_gda_value(m_glom_type, value, std::locale() /* SQL uses the C locale */); //No quotes for numbers.
-      break;
-    }
-    case(TYPE_BOOLEAN):
-    {
-      if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BOOLEAN)
-        str = ( value.get_bool() ? "TRUE" : "FALSE" );
-      else
-        str = "FALSE";
-
-      break;
-    }
-    case(TYPE_IMAGE):
-    {
-      if(value.get_value_type() == Gnome::Gda::VALUE_TYPE_BINARY)
-      {
-        long buffer_size = 0;
-        const gpointer buffer = value.get_binary(buffer_size);
-        if(buffer && buffer_size)
-        {
-          const std::string escaped_binary_data = std::string((char*)buffer, buffer_size);
-          //Now escape that text (to convert \ to \\, for instance):
-          str = glom_escape_text(escaped_binary_data) /* has quotes */ + "::bytea";
-
-          //Use this when libgda unescapes the binary data internally, as it should: 
-          //str = "'" + GlomConversions::get_escaped_binary_data((guint8*)buffer, buffer_size) + "'::bytea";
-        }
-      }
-      else
-        g_warning("Field::sql(): glom_type is TYPE_IMAGE but gda type is not VALUE_TYPE_BINARY");
-
-      break;
-    }
-    default:
-    {
-      str = value.to_string();
-      if(str.empty() && (m_glom_type != Field::TYPE_TEXT))
-        str = "NULL"; //This has probably been handled in get_text_for_gda_value() anyway.
-
-      break;
+      gchar* str = gda_binary_to_string(gdabinary, 0);
+      return (str) ? Glib::ustring(Glib::ScopedPtr<char>(str).get())
+        : Glib::ustring();
     }
   }
+  
+  NumericFormat format_ignored; //Because we use ISO format.
+  return Conversions::get_text_for_gda_value(glom_type, value, std::locale() /* SQL uses the C locale */, format_ignored, true /* ISO standard */);
+}
 
-  return str;
+namespace
+{
+  // Changes the type of a given value
+  void value_reinit(GValue* value, GType g_type)
+  {
+    if(G_IS_VALUE(value) && G_VALUE_TYPE(value) != g_type)
+      g_value_unset(value);
+
+    if(!G_IS_VALUE(value))
+      g_value_init(value, g_type);
+  }
+}
+
+Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, bool& success) const
+{
+  return from_file_format(str, m_glom_type, success);
+}
+
+Gnome::Gda::Value Field::from_file_format(const Glib::ustring& str, glom_field_type glom_type, bool& success)
+{
+  success = true;
+  
+  if(glom_type == TYPE_IMAGE)
+  {
+    GdaBinary* gdabinary = gda_string_to_binary(str.c_str());
+    if(!success)
+      return Gnome::Gda::Value();
+    else
+    {
+      Gnome::Gda::Value value;
+      value_reinit(value.gobj(), GDA_TYPE_BINARY);
+      gda_value_take_binary(value.gobj(), gdabinary);
+      return value; 
+    }
+  }
+  else
+  {
+    NumericFormat format_ignored; //Because we use ISO format.
+    return Conversions::parse_value(glom_type, str, format_ignored, success, true);
+  }
 }
 
 Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
@@ -350,7 +345,7 @@ Glib::ustring Field::sql_find(const Gnome::Gda::Value& value) const
       if(value.is_null())
         return "''"; //We want to ignore the concept of NULL strings, and deal only with empty strings.
       else
-        return ("'%" + value.to_string() + "%'"); //Add single-quotes. Actually escape it.
+        return ("'%" + value.to_string() + "%'"); //Add single-quotes. TODO: Actually escape it.
         
       break;
     }
@@ -372,7 +367,11 @@ Glib::ustring Field::sql_find_operator() const
   {
     case(TYPE_TEXT):
     {
-      return "ILIKE"; //"LIKE"; ILIKE is a postgres extension for locale-dependent case-insensitive matches.
+      ConnectionPool* connection_pool = ConnectionPool::get_instance();
+      if(connection_pool && connection_pool->get_backend())
+        return connection_pool->get_string_find_operator();
+      else
+        return "LIKE"; // Default
       break;
     }
     case(TYPE_DATE):
@@ -390,57 +389,81 @@ Glib::ustring Field::sql_find_operator() const
 
 Glib::ustring Field::get_name() const
 {
-  return m_field_info.get_name();
+  return m_field_info->get_name();
 }
 
 void Field::set_name(const Glib::ustring& value)
 {
-  m_field_info.set_name(value);
+  m_field_info->set_name(value);
 }
 
 bool Field::get_auto_increment() const
 {
-  return m_field_info.get_auto_increment();
+  return m_field_info->get_auto_increment();
 }
 
 void Field::set_auto_increment(bool val)
 {
-  m_field_info.set_auto_increment(val);
+  m_field_info->set_auto_increment(val);
 }
 
 bool Field::get_primary_key() const
 {
-  return m_field_info.get_primary_key();
+  //TODO_gda: return m_field_info->get_primary_key();
+  return m_primary_key;
 }
 
 void Field::set_primary_key(bool val)
 {
-  m_field_info.set_primary_key(val);
+  m_primary_key = val;
+  //TODO_gda: m_field_info->set_primary_key(val);
 }
 
 bool Field::get_unique_key() const
 {
-  return m_field_info.get_unique_key();
+  //TODO_gda: return m_field_info->get_unique_key();
+  return m_unique_key;
 }
 
 void Field::set_unique_key(bool val)
 {
-  m_field_info.set_unique_key(val);
+  //TODO_gda: m_field_info->set_unique_key(val);
+  m_unique_key = val;
 }
 
 Gnome::Gda::Value Field::get_default_value() const
 {
-  return m_field_info.get_default_value();
+  return m_field_info->get_default_value();
 }
 
-void Field::set_default_value(const Gnome::Gda::Value& val)
+void Field::set_default_value(const Gnome::Gda::Value& value)
 {
-  m_field_info.set_default_value(val);
+  // TODO: Allow setting a NULL default value when glom type is invalid?
+
+  // Verify that the value matches the type of the field.
+  GType cur_type = get_gda_type_for_glom_type(get_glom_type());
+  const FieldTypes* pFieldTypes = NULL;
+
+  ConnectionPool* pConnectionPool = ConnectionPool::get_instance();
+  if(pConnectionPool)
+    pFieldTypes = pConnectionPool->get_field_types();
+
+  // Take into account that value might be one of the fallback types
+  if(pFieldTypes)
+  {
+    while(cur_type != value.get_value_type() && cur_type != G_TYPE_NONE)
+      cur_type = pFieldTypes->get_fallback_type_for_gdavaluetype(cur_type);
+  }
+
+  if(value.is_null() || value.get_value_type() == cur_type)
+    m_field_info->set_default_value(value);
+  else
+    g_warning("Field::set_default_value: Cannot set incompatible default value: Default value has type %s, but field has type %s", g_type_name(value.get_value_type()), g_type_name(get_gda_type_for_glom_type(get_glom_type())));
 }
 
 Glib::ustring Field::get_sql_type() const
 {
-  if(false) //See generate_next_auto_increment() //m_field_info.get_auto_increment())
+  if(false) //See get_next_auto_increment_value() //m_field_info->get_auto_increment())
     return "serial"; //See Postgres manual: http://www.postgresql.org/docs/7.4/interactive/datatype.html#DATATYPE-SERIAL. It's actually an integer..
   else
   {
@@ -455,33 +478,73 @@ Glib::ustring Field::get_sql_type() const
       const FieldTypes* pFieldTypes = pConnectionPool->get_field_types();
       if(pFieldTypes)
       {
-        const Gnome::Gda::ValueType fieldType = m_field_info.get_gdatype();
+        const GType fieldType = m_field_info->get_g_type();
         strType = pFieldTypes->get_string_name_for_gdavaluetype(fieldType);
       }
     }
 
     if(strType == "unknowntype")
     {
-      g_warning("Field::get_sql_type(): returning unknowntype for field name=%s , glom_type=%d, gda_type=%d", get_name().c_str(), get_glom_type(), m_field_info.get_gdatype());
+      g_warning("Field::get_sql_type(): returning unknowntype for field name=%s , glom_type=%d, gda_type=%d", get_name().c_str(), get_glom_type(), (int)m_field_info->get_g_type());
     }
 
     return strType;
   }
 }
 
-/// Ignores any part of FieldAttributes that libgda does not properly fill.
-bool Field::field_info_from_database_is_equal(const Gnome::Gda::FieldAttributes& field)
+Glib::ustring Field::get_gda_type_name() const
 {
-  Gnome::Gda::FieldAttributes temp = m_field_info;
-
-  temp.set_auto_increment( field.get_auto_increment() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
-  temp.set_default_value( field.get_default_value() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
-  temp.set_primary_key( field.get_primary_key() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
-
-  return temp == field; 
+  //return g_type_name( get_gda_type_for_glom_type(m_glom_type) );
+  return g_type_name( m_field_info->get_g_type());
 }
 
-Field::glom_field_type Field::get_glom_type_for_gda_type(Gnome::Gda::ValueType gda_type)
+Glib::RefPtr<Gnome::Gda::Holder> Field::get_holder(const Gnome::Gda::Value& value, const Glib::ustring& name) const
+{
+  const Glib::ustring real_name = name.empty() ? get_name() : name;
+
+  const GType gtype = value.get_value_type();
+
+  /*
+  const GType field_type = m_field_info->get_g_type();
+  if(m_field_info->get_g_type() != gtype)
+  {
+    // This currently happens with SQLite for pictures. The value type is
+    // GdaBinary, but the field type is GdaBlob. This is not an optimal
+    // solution this way. We should maybe check fallback types here, or
+    // investigate why the field type is not GdaBinary as well.
+    // Maybe get_gda_type_for_glom_type() should already return fallback
+    // types if necessary.
+    std::cout << "DEBUG: Field::get_holder(): Field type " << g_type_name(field_type) << " and value type " << g_type_name(gtype) << " don't match." << std::endl;
+  }
+  */
+
+  Glib::RefPtr<Gnome::Gda::Holder> holder = Gnome::Gda::Holder::create(gtype, real_name);
+  holder->set_value_as_value(value);
+  return holder;
+}
+
+Glib::ustring Field::get_gda_holder_string(const Glib::ustring& name) const
+{
+  const Glib::ustring real_name = name.empty() ? get_name() : name;
+  return "##" + real_name + "::" + get_gda_type_name();
+}
+
+/// Ignores any part of FieldAttributes that libgda does not properly fill.
+bool Field::field_info_from_database_is_equal(const Glib::RefPtr<const Gnome::Gda::Column>& field)
+{
+  Glib::RefPtr<Gnome::Gda::Column> temp = m_field_info->copy();
+
+  temp->set_auto_increment( field->get_auto_increment() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
+
+  Gnome::Gda::Value value = field->get_default_value();
+  temp->set_default_value(value); //Don't compare this, because the data is incorrect when libgda reads it from the database.
+
+  //TODO_gda: temp->set_primary_key( field->get_primary_key() ); //Don't compare this, because the data is incorrect when libgda reads it from the database.
+
+  return temp->equal(field); 
+}
+
+Field::glom_field_type Field::get_glom_type_for_gda_type(GType gda_type)
 {
   init_map();
 
@@ -494,27 +557,29 @@ Field::glom_field_type Field::get_glom_type_for_gda_type(Gnome::Gda::ValueType g
       result = iterFind->second;
     else
     {
-      // g_warning("FieldType::FieldType(Gnome::Gda::ValueType gda_type): Invalid gda type: %d",  gda_type);
+      // g_warning("FieldType::FieldType(GType gda_type): Invalid gda type: %d",  gda_type);
     }
   }
 
   return result;
 }
 
-Gnome::Gda::ValueType Field::get_gda_type_for_glom_type(Field::glom_field_type glom_type)
+GType Field::get_gda_type_for_glom_type(Field::glom_field_type glom_type)
 {
   init_map();
 
   //Get the ideal gda type used for that glom type;
   type_map_glom_type_to_gda_type::iterator iterFind = m_map_glom_type_to_gda_type.find(glom_type);
-  Gnome::Gda::ValueType ideal_gda_type = Gnome::Gda::VALUE_TYPE_UNKNOWN;
+  GType ideal_gda_type = G_TYPE_NONE;
   if(iterFind != m_map_glom_type_to_gda_type.end())
     ideal_gda_type = iterFind->second;
 
-  if(ideal_gda_type == Gnome::Gda::VALUE_TYPE_UNKNOWN)
+  if(ideal_gda_type == G_TYPE_NONE)
   {
-    g_warning("Field::get_gda_type_for_glom_type(): Returning VALUE_TYPE_UNKNOWN for glom_type=%d", glom_type);
+    g_warning("Field::get_gda_type_for_glom_type(): Returning G_TYPE_NONE for glom_type=%d", glom_type);
   }
+
+  //std::cout << "Field::get_gda_type_for_glom_type(): returning: " << g_type_name(ideal_gda_type) << std::endl;
   
   return ideal_gda_type;
 }
@@ -527,20 +592,20 @@ void Field::init_map()
     //Fill maps.
 
     //Ideals:
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_NUMERIC] = TYPE_NUMERIC;
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_INTEGER] = TYPE_NUMERIC; //Only for "serial" (auto-increment) fields.
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_STRING] = TYPE_TEXT;
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_TIME] = TYPE_TIME;
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_DATE] = TYPE_DATE;
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_BOOLEAN] = TYPE_BOOLEAN;
-    m_map_gda_type_to_glom_type[Gnome::Gda::VALUE_TYPE_BINARY] = TYPE_IMAGE;
+    m_map_gda_type_to_glom_type[GDA_TYPE_NUMERIC] = TYPE_NUMERIC;
+    m_map_gda_type_to_glom_type[G_TYPE_INT] = TYPE_NUMERIC; //Only for "serial" (auto-increment) fields.
+    m_map_gda_type_to_glom_type[G_TYPE_STRING] = TYPE_TEXT;
+    m_map_gda_type_to_glom_type[GDA_TYPE_TIME] = TYPE_TIME;
+    m_map_gda_type_to_glom_type[G_TYPE_DATE] = TYPE_DATE;
+    m_map_gda_type_to_glom_type[G_TYPE_BOOLEAN] = TYPE_BOOLEAN;
+    m_map_gda_type_to_glom_type[GDA_TYPE_BINARY] = TYPE_IMAGE;
 
-    m_map_glom_type_to_gda_type[TYPE_NUMERIC] = Gnome::Gda::VALUE_TYPE_NUMERIC;
-    m_map_glom_type_to_gda_type[TYPE_TEXT] = Gnome::Gda::VALUE_TYPE_STRING;
-    m_map_glom_type_to_gda_type[TYPE_TIME] = Gnome::Gda::VALUE_TYPE_TIME;
-    m_map_glom_type_to_gda_type[TYPE_DATE] = Gnome::Gda::VALUE_TYPE_DATE;
-    m_map_glom_type_to_gda_type[TYPE_BOOLEAN] = Gnome::Gda::VALUE_TYPE_BOOLEAN;
-    m_map_glom_type_to_gda_type[TYPE_IMAGE] = Gnome::Gda::VALUE_TYPE_BINARY;
+    m_map_glom_type_to_gda_type[TYPE_NUMERIC] = GDA_TYPE_NUMERIC;
+    m_map_glom_type_to_gda_type[TYPE_TEXT] = G_TYPE_STRING;
+    m_map_glom_type_to_gda_type[TYPE_TIME] = GDA_TYPE_TIME;
+    m_map_glom_type_to_gda_type[TYPE_DATE] = G_TYPE_DATE;
+    m_map_glom_type_to_gda_type[TYPE_BOOLEAN] = G_TYPE_BOOLEAN;
+    m_map_glom_type_to_gda_type[TYPE_IMAGE] = GDA_TYPE_BINARY;
 
     m_map_type_names_ui[TYPE_INVALID] = _("Invalid");
     m_map_type_names_ui[TYPE_NUMERIC] = _("Number");
@@ -570,8 +635,8 @@ void Field::init_map()
     list_conversions.clear();
     list_conversions.push_back(Field::TYPE_BOOLEAN);
     list_conversions.push_back(Field::TYPE_TEXT);
-    list_conversions.push_back(Field::TYPE_DATE);
-    list_conversions.push_back(Field::TYPE_TIME);
+    //to_date(numeric) was supported in 8.2 but not in 8.3: list_conversions.push_back(Field::TYPE_DATE);
+    //to_timestamp(numeric) was supported in 8.2 but not in 8.3: list_conversions.push_back(Field::TYPE_TIME);
     m_map_conversions[Field::TYPE_NUMERIC] = list_conversions;
 
     //Text:
@@ -586,15 +651,15 @@ void Field::init_map()
     list_conversions.clear();
     list_conversions.push_back(Field::TYPE_TEXT);
     list_conversions.push_back(Field::TYPE_NUMERIC);
-    list_conversions.push_back(Field::TYPE_DATE);
-    list_conversions.push_back(Field::TYPE_TIME);
+    //to_timestamp(numeric) was supported in 8.2 but not in 8.3: list_conversions.push_back(Field::TYPE_DATE);
+    //to_timestamp(numeric) was supported in 8.2 but not in 8.3: list_conversions.push_back(Field::TYPE_TIME);
     m_map_conversions[Field::TYPE_BOOLEAN] = list_conversions;
 
     //Date:
     list_conversions.clear();
     list_conversions.push_back(Field::TYPE_TEXT);
-    list_conversions.push_back(Field::TYPE_NUMERIC);
-    list_conversions.push_back(Field::TYPE_BOOLEAN);
+    //to_number(textcat()) was supported in 8.2 but not in 8.3: list_conversions.push_back(Field::TYPE_NUMERIC);
+    //to_number(textcat()) was supported in 8.2 but not in 8.3: list_conversions.push_back(Field::TYPE_BOOLEAN);
     m_map_conversions[Field::TYPE_DATE] = list_conversions;
 
     //Time:
@@ -740,3 +805,5 @@ bool Field::get_visible() const
 {
   return m_visible;
 }
+
+} //namespace Glom
