@@ -21,6 +21,8 @@
 
 #include "config.h" // For HAVE_STRPTIME
 
+#include <libgda/gda-blob-op.h> // For gda_blob_op_read_all()
+
 #include "glomconversions.h"
 #include <glom/libglom/connectionpool.h>
 #include <glom/libglom/utils.h>
@@ -35,6 +37,24 @@
 #include <iomanip>
 #include <string.h> // for strlen, memset, strcmp
 
+namespace
+{
+  char hextochar(guint8 hex)
+  {
+    if(hex < 10) return '0' + hex;
+    if(hex < 16) return 'a' - 10 + hex;
+    return '\0';
+  }
+
+  guint8 chartohex(char c)
+  {
+    if(c >= '0' && c <= '9') return c - '0';
+    if(c >= 'a' && c <= 'f') return c - 'a' + 10;
+    if(c >= 'A' && c <= 'F') return c - 'A' + 10;
+    return 0;
+  }
+}
+
 namespace Glom
 {
 
@@ -272,6 +292,12 @@ double Conversions::get_double_for_gda_value_numeric(const Gnome::Gda::Value& va
 {
   if(value.get_value_type() != GDA_TYPE_NUMERIC)
   {
+    // Note that in case the database system does not support GdaNumeric
+    // (such as sqlite) we fall back to double (see
+    // FieldTypes::get_string_name_for_gdavaluetype), so try this as well.
+    if(value.get_value_type() == G_TYPE_DOUBLE)
+      return value.get_double();
+
     std::cerr << "Conversions::get_double_for_gda_value_numeric(): expected NUMERIC but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
     return 0;
   }
@@ -295,21 +321,42 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
 {
   if(value.is_null()) //The type can be null for any of the actual field types.
   {
-    return "";
+    return Glib::ustring();
   }
 
-  if( (glom_type == Field::TYPE_DATE) && (value.get_value_type() == G_TYPE_DATE))
+  if(glom_type == Field::TYPE_DATE)
   {
-    Glib::Date gda_date = value.get_date();
-    //Gnome::Gda::Date gda_date = value.get_date();
-
-    //tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     tm the_c_time;
     memset(&the_c_time, 0, sizeof(the_c_time));
 
-    the_c_time.tm_year = gda_date.get_year() - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
-    the_c_time.tm_mon = gda_date.get_month() - 1; //C months start at 0.
-    the_c_time.tm_mday = gda_date.get_day(); //starts at 1
+    if(value.get_value_type() == G_TYPE_STRING)
+    {
+      // If a date is contained in a string type instead of a date type,
+      // which can happen if the database system does not support dates
+      // natively, then the string representation is always in ISO format.
+      bool success;
+      the_c_time = parse_date(value.get_string(), std::locale::classic(), success);
+      if(!success)
+        std::cerr << "Conversions::get_text_for_gda_value(): Failed to convert string-represented date value" << std::endl;
+    }
+    else if(value.get_value_type() == G_TYPE_DATE)
+    {
+      Glib::Date gda_date = value.get_date();
+      //Gnome::Gda::Date gda_date = value.get_date();
+
+      //tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+
+      the_c_time.tm_year = gda_date.get_year() - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
+      the_c_time.tm_mon = gda_date.get_month() - 1; //C months start at 0.
+      the_c_time.tm_mday = gda_date.get_day(); //starts at 1
+    }
+    else
+    {
+      std::cerr << "Conversions::get_text_for_gda_value(): glom field type is DATE but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
+
+      // Default
+      the_c_time.tm_mday = 1;
+    }
 
     return format_date(the_c_time, locale, iso_format);
 
@@ -319,23 +366,40 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
     return date.format_string("%x"); //%x means "is replaced by the locale's appropriate date representation".
     */
   }
-  else if((glom_type == Field::TYPE_TIME) && (value.get_value_type() == GDA_TYPE_TIME))
+  else if(glom_type == Field::TYPE_TIME)
   {
-    Gnome::Gda::Time gda_time = value.get_time();
-
-    //tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
     tm the_c_time;
     memset(&the_c_time, 0, sizeof(the_c_time));
+    if(value.get_value_type() == G_TYPE_STRING)
+    {
+      // If a time is contained in a string type instead of a gda time type,
+      // which can happen if the database system does not support times
+      // natively, then the string representation is always in ISO format.
+      bool success;
+      the_c_time = parse_time(value.get_string(), std::locale::classic(), success);
+      if(!success)
+        std::cerr << "Conversions::get_text_for_gda_value(): Failed to convert string-represented time value" << std::endl;
+    }
+    else if(value.get_value_type() == GDA_TYPE_TIME)
+    {
+      Gnome::Gda::Time gda_time = value.get_time();
 
-    the_c_time.tm_hour = gda_time.hour;
-    the_c_time.tm_min = gda_time.minute;
-    the_c_time.tm_sec = gda_time.second;
+      //tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+
+      the_c_time.tm_hour = gda_time.hour;
+      the_c_time.tm_min = gda_time.minute;
+      the_c_time.tm_sec = gda_time.second;
+    }
+    else
+    {
+      std::cerr << "Conversions::get_text_for_gda_value(): glom field type is TIME but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
+    }
 
     return format_time(the_c_time, locale, iso_format);
   }
   else if(glom_type == Field::TYPE_NUMERIC)
   {
-    if(value.get_value_type() != GDA_TYPE_NUMERIC)
+    if(value.get_value_type() != GDA_TYPE_NUMERIC && value.get_value_type() != G_TYPE_DOUBLE)
     {
       std::cerr << "Conversions::get_text_for_gda_value(): glom field type is NUMERIC but GdaValue type is: " << g_type_name(value.get_value_type()) << std::endl;
       return value.to_string();
@@ -382,7 +446,7 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
 
     if(locale == std::locale("") /* The user's current locale */)
     {
-      //Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
+      // Converts from the user's current locale to utf8. I would prefer a generic conversion from any locale,
       // but there is no such function, and it's hard to do because I don't know how to get the encoding name from the std::locale()
 #ifdef GLIBMM_EXCEPTIONS_ENABLED
       text = Glib::locale_to_utf8(text);
@@ -392,22 +456,30 @@ Glib::ustring Conversions::get_text_for_gda_value(Field::glom_field_type glom_ty
 #endif
     }
 
+    //std::cout << "DEBUG: Conversions::get_text_for_gda_value(): number=" << number << ", text=" << text << std::endl;
     return text; //Do something like Glib::locale_to_utf(), but with the specified locale instead of the current locale.
   }
-  else if (glom_type == Field::TYPE_TEXT)
+  else if(glom_type == Field::TYPE_TEXT)
   {
      return value.get_string();
   }
+  else if (glom_type == Field::TYPE_BOOLEAN)
+  {
+    //This is used only by Field::to_file_format(),
+    //and should never be shown in the UI.
+    if(G_VALUE_TYPE(value.gobj()) == G_TYPE_BOOLEAN)
+      return (value.get_boolean() ? "TRUE" : "FALSE" );
+    else
+      return "FALSE";
+  }
   else if (glom_type == Field::TYPE_IMAGE)
   {
-    //Return the binary-as-escaped-text format, suitable for use in the document. 
-    std::string result;
-    long buffer_length;
-    const guchar* buffer = value.get_binary(buffer_length);
-    if(buffer && buffer_length > 0)
-      result = Conversions::get_escaped_binary_data((guint8*)buffer, buffer_length);
-
-    return result;
+    //This function is only used for :
+    //- UI-visible strings, but images should never be shown as text in the UI. 
+    //- Values in SQL queries, but we only do that for clauses (where/sort/order) 
+    //  which should never use image values.
+    std::cerr << "Conversions::get_text_for_gda_value(): Unexpected TYPE_IMAGE field type: " << glom_type << std::endl;
+    return Glib::ustring();
   }
   else
   {
@@ -421,13 +493,12 @@ Gnome::Gda::Value Conversions::parse_value(double number)
   //This is just a way to get a NUMERIC Gnome::Gda::Value from a numeric type:
   //Try to parse the inputted number, according to the current locale.
 
-  GdaNumeric gda_numeric = {0, 0, 0};
+  GdaNumeric gda_numeric = {0, 0, 0, 0};
 
   //Then generate a canonical representation of the number:
   std::stringstream clocale_stream;
   clocale_stream.imbue( std::locale::classic() ); //The C locale.
   clocale_stream << number;
-  clocale_stream << std::setprecision(STRINGSTREAM_PRECISION_DEFAULT); //Avoid the e syntax. Normally this happens afer 7 digits, with loss of precision. TODO: Handle more.
   Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
 
   //TODO: What about the precision and width values?
@@ -482,7 +553,7 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
       the_c_time = parse_time(text, success);
     }
 
-    Gnome::Gda::Time gda_time = {0, 0, 0, 0};
+    Gnome::Gda::Time gda_time = {0, 0, 0, 0, 0};
     gda_time.hour = the_c_time.tm_hour;
     gda_time.minute = the_c_time.tm_min;
     gda_time.second = the_c_time.tm_sec;
@@ -512,16 +583,21 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
     the_stream.imbue( the_locale ); //Parse it as per the specified locale.
     the_stream.str(text_to_parse); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
     double the_number = 0;
-    the_stream >> the_number;  //TODO: Does this throw any exception if the text is an invalid time?
+    the_stream >> the_number;  //TODO: Does this throw any exception if the text is an invalid number?
 
-    GdaNumeric gda_numeric = {0, 0, 0};
+    //std::cout << "DEBUG: Conversions::parse_value(): text=" << text_to_parse << ", number=" << the_number << std::endl;
+    
+
+    GdaNumeric gda_numeric = {0, 0, 0, 0};
 
     //Then generate a canonical representation of the number:
 
     std::stringstream clocale_stream;
     clocale_stream.imbue( std::locale::classic() ); //The C locale.
+    clocale_stream << std::setprecision(STRINGSTREAM_PRECISION_DEFAULT); //Avoid the e syntax. Normally this happens afer 7 digits, with loss of precision. TODO: Handle more.
     clocale_stream << the_number;
-    Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
+    const Glib::ustring number_canonical_text = clocale_stream.str(); //Avoid << because it does implicit character conversion (though that might not be a problem here. Not sure). murrayc
+    //std::cout << "  DEBUG: number_canonical_text=" << number_canonical_text << std::endl;  
 
     //TODO: What about the precision and width values?
     /* From the postgres docs:
@@ -537,25 +613,14 @@ Gnome::Gda::Value Conversions::parse_value(Field::glom_field_type glom_type, con
   else if(glom_type == Field::TYPE_BOOLEAN)
   {
     success = true;
-    return Gnome::Gda::Value( (text == "TRUE" ? true : false) ); //TODO: Internationalize this, but it should never be used anyway.
+    return Gnome::Gda::Value( (text.uppercase() == "TRUE" ? true : false) ); //TODO: Internationalize this, but it should never be used anyway.
   }
   else if(glom_type == Field::TYPE_IMAGE)
   {
-    //We assume that the text is the same (escaped text) format that we use in the document when saving images:
-    //(The SQL format).
-    Gnome::Gda::Value result;
-
-    size_t buffer_binary_length = 0;
-    guchar* buffer_binary = Glom_PQunescapeBytea((const guchar*)text.c_str() /* must be null-terminated */, &buffer_binary_length); //freed by us later.
-    if(buffer_binary)
-    {
-      result.set(buffer_binary, buffer_binary_length);
-      success = true;
-
-      free(buffer_binary);
-
-      return result;
-    }
+    //This function is only used for :
+    //- UI-visible strings, but images should never be entered as text in the UI. 
+    std::cerr << "Conversions::parse_value(): Unexpected TYPE_IMAGE field type: " << glom_type << std::endl;
+    return Gnome::Gda::Value();
   }
 
   success = true;
@@ -888,194 +953,6 @@ Gnome::Gda::Value Conversions::get_example_value(Field::glom_field_type field_ty
   }
 }
 
-
-unsigned char *
-Glom_PQescapeBytea(const unsigned char *bintext, size_t binlen, size_t *bytealen)
-{
-  const unsigned char *vp;
-  unsigned char *rp;
-  unsigned char *result;
-  size_t    i;
-  size_t    len;
-
-  /*
-   * empty string has 1 char ('\0')
-   */
-  len = 1;
-
-  vp = bintext;
-  for (i = binlen; i > 0; i--, vp++)
-  {
-    if (*vp < 0x20 || *vp > 0x7e)
-      len += 5;     /* '5' is for '\\ooo' */
-    else if (*vp == '\'')
-      len += 2;
-    else if (*vp == '\\')
-      len += 4;
-    else
-      len++;
-  }
-
-  rp = result = (unsigned char *) malloc(len);
-  if (rp == NULL)
-    return NULL;
-
-  vp = bintext;
-  *bytealen = len;
-
-  for (i = binlen; i > 0; i--, vp++)
-  {
-    if (*vp < 0x20 || *vp > 0x7e)
-    {
-      (void) sprintf((char*)rp, "\\\\%03o", *vp);
-      rp += 5;
-    }
-    else if (*vp == '\'')
-    {
-      rp[0] = '\\';
-      rp[1] = '\'';
-      rp += 2;
-    }
-    else if (*vp == '\\')
-    {
-      rp[0] = '\\';
-      rp[1] = '\\';
-      rp[2] = '\\';
-      rp[3] = '\\';
-      rp += 4;
-    }
-    else
-      *rp++ = *vp;
-  }
-  *rp = '\0';
-
-  return result;
-}
-
-#define ISFIRSTOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '3')
-#define ISOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '7')
-#define OCTVAL(CH) ((CH) - '0')
-
-unsigned char *
-Glom_PQunescapeBytea(const unsigned char *strtext, size_t *retbuflen)
-{
-  size_t    strtextlen,
-        buflen;
-  unsigned char *buffer,
-         *tmpbuf;
-  size_t    i,
-        j;
-
-  if (strtext == NULL)
-    return NULL;
-
-  strtextlen = strlen((const char*)strtext);
-
-  /*
-   * Length of input is max length of output, but add one to avoid
-   * unportable malloc(0) if input is zero-length.
-   */
-  buffer = (unsigned char *) malloc(strtextlen + 1);
-  if (buffer == NULL)
-    return NULL;
-
-  for (i = j = 0; i < strtextlen;)
-  {
-    switch (strtext[i])
-    {
-      case '\\':
-        i++;
-        if (strtext[i] == '\\')
-          buffer[j++] = strtext[i++];
-        else
-        {
-          if ((ISFIRSTOCTDIGIT(strtext[i])) &&
-            (ISOCTDIGIT(strtext[i + 1])) &&
-            (ISOCTDIGIT(strtext[i + 2])))
-          {
-            int     byte;
-
-            byte = OCTVAL(strtext[i++]);
-            byte = (byte << 3) + OCTVAL(strtext[i++]);
-            byte = (byte << 3) + OCTVAL(strtext[i++]);
-            buffer[j++] = byte;
-          }
-        }
-
-        /*
-         * Note: if we see '\' followed by something that isn't a
-         * recognized escape sequence, we loop around having done
-         * nothing except advance i.  Therefore the something will
-         * be emitted as ordinary data on the next cycle. Corner
-         * case: '\' at end of string will just be discarded.
-         */
-        break;
-
-      default:
-        buffer[j++] = strtext[i++];
-        break;
-    }
-  }
-  buflen = j;         /* buflen is the length of the dequoted
-                 * data */
-
-  /* Shrink the buffer to be no larger than necessary */
-  /* +1 avoids unportable behavior when buflen==0 */
-  tmpbuf = (unsigned char*)realloc(buffer, buflen + 1);
-
-  /* It would only be a very brain-dead realloc that could fail, but... */
-  if (!tmpbuf)
-  {
-    free(buffer);
-    return NULL;
-  }
-
-  *retbuflen = buflen;
-  return tmpbuf;
-}
-
-Glib::ustring Conversions::get_escaped_binary_data(guint8* buffer, size_t buffer_size)
-{
-  //g_warning("Conversions::get_escaped_binary_data: debug: buffer ");
-  //for(int i = 0; i < 10; ++i)
-  //  g_warning("%02X (%c), ", (guint8)buffer[i], buffer[i]);
-
-  //TODO: Performance: Preallocate a string of the appropriate size.
-  //Use an output parameter instead of copying it during return.
-
-  Glib::ustring result;
-
-  /* This should be much faster, and more correct, but it gives an error - "unterminated quoted string at or near: "
-  size_t escaped_length = 0;
-  guchar* escaped =  Glom_PQescapeBytea(buffer, buffer_size, &escaped_length);
-
-  if(escaped && escaped_length)
-  {
-    result = std::string((char*)escaped, escaped_length);
-    free(escaped);
-  }
-  */
-
-
-  if(buffer && buffer_size)
-  {
-    guint8* buffer_end = buffer + buffer_size;
-    char byte_as_octal[4]; //3 digits, and a null terminator
-
-    for(guint8* pos = buffer; pos < buffer_end; ++pos)
-    {
-      sprintf(byte_as_octal, "%03o", *pos); //Format as octal with 3 digits.
-      byte_as_octal[3] = 0;
-
-//      g_warning("byte=%d, as_hex=%s", *pos, byte_as_octal);      
-
-      result += Glib::ustring("\\") + byte_as_octal; //Note that this string must be escaped once more to convert \ ("\\") into \\ ("\\\\").
-    }
-  }
-
-  return result;
-}
-
 Gnome::Gda::Value Conversions::convert_value(const Gnome::Gda::Value& value, Field::glom_field_type target_glom_type)
 {
   const GType gvalue_type = value.get_value_type();
@@ -1105,10 +982,29 @@ Glib::RefPtr<Gdk::Pixbuf> Conversions::get_pixbuf_for_gda_value(const Gnome::Gda
 {
   Glib::RefPtr<Gdk::Pixbuf> result;
 
-  if(value.get_value_type() == GDA_TYPE_BINARY)
+  if(value.get_value_type() == GDA_TYPE_BINARY || value.get_value_type() == GDA_TYPE_BLOB)
   {
-    glong buffer_binary_length = 0;
-    gconstpointer buffer_binary = value.get_binary(buffer_binary_length);
+    glong buffer_binary_length;
+    gconstpointer buffer_binary;
+    if(value.get_value_type() == GDA_TYPE_BLOB)
+    {
+      const GdaBlob* blob = value.get_blob();
+      if(gda_blob_op_read_all(blob->op, const_cast<GdaBlob*>(blob)))
+      {
+        buffer_binary_length = blob->data.binary_length;
+        buffer_binary = blob->data.data;
+      }
+      else
+      {
+        buffer_binary_length = 0;
+        buffer_binary = NULL;
+        g_warning("Conversions::get_pixbuf_for_gda_value(): Failed to read BLOB data");
+      }
+    }
+    else
+    {
+      buffer_binary = value.get_binary(buffer_binary_length);
+    }
 
     /* Note that this is regular binary data, not escaped text representing the binary data: */
     if(buffer_binary && buffer_binary_length)
